module TestRunner
  module Assert
    module Checks
      require_relative "checks/registry"
      extend Registry

      register :equals do |check, object|
        obj_inspect = Assert.inspect object

        if Assert.inspect(check.subject) == obj_inspect and not check.negated?
          fail_message = "equal other object (no difference in #inspect output)"
        else
          fail_message = "equal #{obj_inspect}"
        end

        check.fail fail_message do check.subject == object end
      end

      register :included_in do |check, list|
        check.fail "be included in #{Assert.inspect list}" do
          list.include? check.subject
        end
      end

      register :kind_of do |check, type|
        check.fail "be a kind of #{type}" do
          check.subject.kind_of? type
        end
      end

      register :matches do |check, matcher|
        check.fail "match #{Assert.inspect matcher}" do
          check.subject.match matcher
        end
      end

      register :predicate do |check, argument|
        name, *args = argument
        method_name =  "#{name}?"
        method = check.subject.method method_name

        if method.arity == 0
          expect_truth = args.all?
          message = "be #{name}"
          result = check.subject.public_send method_name
          result = result ^ !expect_truth
        else
          result = check.subject.public_send method_name, *args
          args = args.map &Assert.method(:inspect)
          message = "#{name} #{args * ', '}"
        end

        check.fail message do result end
      end

      register :raises do |check, error_type|
        check.expect_error
        check.fail "be a #{Assert.inspect error_type}" do
          check.subject.is_a? error_type
        end
      end

      register :responds_to do |check, method_name|
        check.fail "respond to ##{method_name}" do
          check.subject.respond_to? method_name
        end
      end

      register :truthy do |check, arg|
        check.fail "be #{arg ? "truthy" : "falsey"}" do
          !arg ^ check.subject
        end
      end
    end
  end
end
