module TestRunner
  module Assert
    Error ||= Class.new StandardError

    class AssertionFailed < Error
      attr :assertion

      def initialize assertion
        @assertion = assertion
      end

      def backtrace
        assertion.trace.map &:to_s
      end

      def to_s
        failures = assertion.fails.map do |failed_check|
          failed_check.fail_message
        end
        if failures.size > 1
          "Assertion failure:\n\n    * #{failures * "\n    * "}\n"
        else
          "Assertion failure: #{failures * ", "}"
        end
      end
    end

    class MissingCheck < Error
      attr :check_name

      def initialize check_name
        @check_name = check_name
      end

      def to_s
        "could not resolve check #{check_name.inspect}"
      end
    end

    class NothingRaised < Error
      def to_s
        "expected subject block to raise an error"
      end
    end
  end
end
